﻿using GazBooksLibrary.DAO;
using GazBooksLibrary.Model;
using System;
using System.Collections.Generic;
using System.ServiceModel;

namespace GazBooksLibrary.BLL
{
	[ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
	public class EmprestimoService : IEmprestimoService
	{
		private EmprestimoDAO dao = new EmprestimoDAO();
		private LivroDAO ldao = new LivroDAO();
		private PeriodicoDAO pdao = new PeriodicoDAO();
		private ClienteDAO cdao = new ClienteDAO();
		
		public bool InserirEmprestimoPeriodico(UInt32 id_periodico, UInt32 id_cliente, String data_emprestimo, String data_devolucao)
		{
			Cliente c;
			lock (cdao)
			{
				c = cdao.BuscarCliente(id_cliente);
			}
			List<Emprestimo> emprestimos = dao.BuscarEmprestimosPorCliente(id_cliente);
			if (c == null || (emprestimos != null && emprestimos.Count + 1 > c.LimiteEmprestimos))
			{
				return false;
			}
			Periodico p;
			lock (pdao)
			{
				p = pdao.BuscarPeriodicoPorID(id_periodico);
			}
			if (p == null || p.Qtde == 0)
			{
				return false;
			}
			lock (dao)
			{
				dao.InserirNovoEmprestimo(id_periodico, id_cliente, data_emprestimo, data_devolucao, 1);
				pdao.AtualizarQuantidade(id_periodico, ((UInt16) (p.Qtde - 1)));
			}
			return true;
		}

		public bool InserirEmprestimo(UInt32 id_livro, UInt32 id_cliente, String data_emprestimo, String data_devolucao)
		{
			Cliente c;
			lock (cdao)
			{
				c = cdao.BuscarCliente(id_cliente);
			}
			List<Emprestimo> emprestimos = dao.BuscarEmprestimosPorCliente(id_cliente);
			if (c == null || (emprestimos != null && emprestimos.Count + 1 > c.LimiteEmprestimos))
			{
				return false;
			}
			Livro l;
			lock (ldao)
			{
				l = ldao.BuscarLivroPorID(id_livro);
			}
			if (l == null || l.Qtde == 0)
			{
				return false;
			}
			lock (dao)
			{
				dao.InserirNovoEmprestimo(id_livro, id_cliente, data_emprestimo, data_devolucao, 2);
				ldao.AtualizarQuantidade(id_livro, (UInt16) (l.Qtde - 1));
			}
			return true;
		}

		public void RegistrarDevolucaoPeriodico(UInt32 id_emprestimo)
		{
			Emprestimo e;
			lock(dao)
			{
				e = dao.BuscarEmprestimoPorID(id_emprestimo, 1);
			}
			if (e == null)
			{
				return;
			}
			lock (dao)
			{
				dao.RegistrarDevolucao(id_emprestimo, 1);
			}
			lock (pdao)
			{
				Periodico p = pdao.BuscarPeriodicoPorID(e.ID_Obra);
				pdao.AtualizarQuantidade(p.ID, (UInt16) (p.Qtde + 1));
			}
		}

		public void RegistrarDevolucao(UInt32 id_emprestimo)
		{
			Emprestimo e;
			lock(dao)
			{
				e = dao.BuscarEmprestimoPorID(id_emprestimo, 2);
			}
			if (e == null)
			{
				return;
			}
			lock (dao)
			{
				dao.RegistrarDevolucao(id_emprestimo, 2);
			}
			lock (ldao)
			{
				Livro l = ldao.BuscarLivroPorID(e.ID_Obra);
				ldao.AtualizarQuantidade(l.ID, (UInt16)(l.Qtde + 1));
			}
		}

		public List<Emprestimo> BuscarEmprestimosPorCliente(UInt32 id_cliente)
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosPorCliente(id_cliente);
			}
		}

		public List<Emprestimo> BuscarEmprestimosPorPeriodico(UInt32 id_periodico)
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosPorObra(id_periodico, 1);
			}
		}

		public List<Emprestimo> BuscarEmprestimosPorLivro(UInt32 id_livro)
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosPorObra(id_livro, 2);
			}
		}

		public List<Emprestimo> BuscarEmprestimosPorData(String data)
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosPorData(data);
			}
		}

		public List<Emprestimo> BuscarEmprestimosDevolvidos()
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosDevolvidos();
			}
		}

		public List<Emprestimo> BuscarEmprestimosPendentes()
		{
			lock (dao)
			{
				return dao.BuscarEmprestimosPendentes();
			}
		}
	}
}
