﻿using GazBooksLibrary.Model;
using System;
using System.Collections.Generic;
using System.Data.SQLite;

namespace GazBooksLibrary.DAO
{
    public class ClienteDAO
    {
        private String src = @"Data Source=C:\Users\Bruno\GazBooks\GazBooks.db";
        //private String src = @"Data Source=C:\Users\Cida\Desktop\gazbooks\GazBooks.db";

        public void InserirCliente(String nome, String cpf, UInt16 limite)
        {
            String query = "insert into clientes (nome, cpf, limite_emprestimos) values (@nome, @cpf, @limite)";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@nome", nome);
            cmd.Parameters.AddWithValue("@cpf", cpf);
            cmd.Parameters.AddWithValue("@limite", limite);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void DeletarCliente(String cpf)
        {
            String query = "delete from clientes where cpf like @cpf";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@cpf", cpf);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void AlterarNome(String cpf, String nomeNovo)
        {
            String query = "update clientes set nome = @nome where cpf like @cpf";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@nome", nomeNovo);
            cmd.Parameters.AddWithValue("@cpf", cpf);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void AlterarLimite(String cpf, UInt16 novoLimite)
        {
            String query = "update clientes set limite_emprestimos = @limite where cpf like @cpf";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@limite", novoLimite);
            cmd.Parameters.AddWithValue("@cpf", cpf);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        private List<Cliente> BuscarClientesFlex(SQLiteCommand cmd)
        {
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            List<Cliente> clientes = null;
            using (var reader = cmd.ExecuteReader())
            {
                if (!reader.HasRows)
                {
                    return null;
                }
                clientes = new List<Cliente>();
                while (reader.Read())
                {
                    Cliente c = new Cliente();
                    c.ID = (UInt32)reader.GetInt32(0);
                    c.Nome = reader.GetString(1);
                    c.CPF = reader.GetString(2);
                    c.LimiteEmprestimos = (UInt16)reader.GetInt16(3);
                    clientes.Add(c);
                }
            }
            cmd.Dispose();
            con.Close();
            return clientes;
        }

        public List<Cliente> BuscarClientes(String nome)
        {
            String query = "select * from clientes where nome like @nome";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@nome", "%" + nome + "%");
            return BuscarClientesFlex(cmd);
        }

        public Cliente BuscarCliente(UInt32 id)
        {
            String query = "select * from clientes where id = @id";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id);
            List<Cliente> clientes = BuscarClientesFlex(cmd);
            return clientes == null ? null : clientes[0];
        }
    }
}
