﻿using GazBooksLibrary.Model;
using System;
using System.Collections.Generic;
using System.Data.SQLite;

namespace GazBooksLibrary.DAO
{
    public class EmprestimoDAO
    {
        private String src = @"Data Source=C:\Users\Bruno\GazBooks\GazBooks.db";
        //private String src = @"Data Source=C:\Users\Cida\Desktop\gazbooks\GazBooks.db";

        public void InserirNovoEmprestimo(UInt32 id_obra, UInt32 id_cliente, String data_emprestimo, String data_devolucao, UInt16 tipo_emprestimo)
        {
            String query = null;
            if (tipo_emprestimo == 1)
            {
                query = "insert into emprestimos_periodicos (id_cliente, id_periodico, data_emprestimo, data_devolucao, devolvido) values (@id_cliente, @id_obra, @data_emprestimo, @data_devolucao, @devolvido)";
            }
            else
            {
                query = "insert into emprestimos (id_cliente, id_livro, data_emprestimo, data_devolucao, devolvido) values (@id_cliente, @id_obra, @data_emprestimo, @data_devolucao, @devolvido)";
            }
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id_cliente", id_cliente);
            cmd.Parameters.AddWithValue("@id_obra", id_obra);
            cmd.Parameters.AddWithValue("@data_emprestimo", data_emprestimo);
            cmd.Parameters.AddWithValue("@data_devolucao", data_devolucao);
            cmd.Parameters.AddWithValue("@devolvido", false);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void RegistrarDevolucao(UInt32 id_emprestimo, UInt16 tipo_emprestimo)
        {
            String query = null;
            if (tipo_emprestimo == 1)
            {
                query = "update emprestimos_periodicos set devolvido = 1 where id = @id";
            }
            else
            {
                query = "update emprestimos set devolvido = 1 where id = @id";
            }
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id_emprestimo);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void DeletarEmprestimo(UInt32 id_emprestimo, UInt16 tipo_emprestimo)
        {
            String query = null;
            if (tipo_emprestimo == 1)
            {
                query = "delete from emprestimos_periodicos where id = @id";
            }
            else
            {
                query = "delete from emprestimos where id = @id";
            }
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id_emprestimo);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        private List<Emprestimo> BuscarEmprestimosFlex(SQLiteCommand cmd, UInt16 tipo_emprestimo)
        {
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            List<Emprestimo> emprestimos = null;
            using (var reader = cmd.ExecuteReader())
            {
                if (!reader.HasRows)
                {
                    return null;
                }
                emprestimos = new List<Emprestimo>();
                while (reader.Read())
                {
                    Emprestimo e = new Emprestimo();
                    e.ID = (UInt32)reader.GetInt32(0);
                    e.ID_Cliente = (UInt32)reader.GetInt32(1);
                    e.ID_Obra = (UInt32)reader.GetInt32(2);
                    e.Tipo_Emprestimo = tipo_emprestimo;
                    e.Data_Emprestimo = reader.GetString(3);
                    e.Data_Devolucao = reader.GetString(4);
                    e.Devolvido = reader.GetBoolean(5);
                    emprestimos.Add(e);
                }
            }
            cmd.Dispose();
            con.Close();
            return emprestimos;
        }

        public List<Emprestimo> BuscarEmprestimosPorCliente(UInt32 id_cliente)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from emprestimos where id_cliente = @id");
            cmd.Parameters.AddWithValue("@id", id_cliente);
            List<Emprestimo> emprestimos = BuscarEmprestimosFlex(cmd, 2);
            cmd = new SQLiteCommand("select * from emprestimos_periodicos where id_cliente = @id");
            cmd.Parameters.AddWithValue("@id", id_cliente);
            List<Emprestimo> emprestimosPeriodicos = BuscarEmprestimosFlex(cmd, 1);
            if (emprestimos != null)
            {
                if (emprestimosPeriodicos != null)
                {
                    emprestimos.AddRange(emprestimosPeriodicos);
                }
            }
            else if (emprestimosPeriodicos != null)
            {
                emprestimos = new List<Emprestimo>(emprestimosPeriodicos.Count);
                emprestimos.AddRange(emprestimosPeriodicos);
            }
            return emprestimos;
        }

        public Emprestimo BuscarEmprestimoPorID(UInt32 id, UInt16 tipo_emprestimo)
        {
            String query;
            if (tipo_emprestimo == 2)
            {
                query = "select * from emprestimos where id = @id";
            }
            else
            {
                query = "select * from emprestimos_periodicos where id = @id";
            }
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id);
            List<Emprestimo> emprestimos = BuscarEmprestimosFlex(cmd, tipo_emprestimo);
            return emprestimos == null ? null : emprestimos[0];
        }

        public List<Emprestimo> BuscarEmprestimosPorObra(UInt32 id_obra, UInt16 tipo_emprestimo)
        {
            String query = null;
            if (tipo_emprestimo == 1)
            {
                query = "select * from emprestimos_periodicos where id_periodico = @id";
            }
            else
            {
                query = "select * from emprestimos where id_livro = @id";
            }
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id_obra);
            return BuscarEmprestimosFlex(cmd, tipo_emprestimo);
        }

        public List<Emprestimo> BuscarEmprestimosPorData(String data)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from emprestimos where data_emprestimo like @data");
            cmd.Parameters.AddWithValue("@data", data);
            List<Emprestimo> emprestimos = BuscarEmprestimosFlex(cmd, 2);
            cmd = new SQLiteCommand("select * from emprestimos_periodicos where data_emprestimo like @data");
            cmd.Parameters.AddWithValue("@data", data);
            List<Emprestimo> emprestimosPeriodicos = BuscarEmprestimosFlex(cmd, 1);
            if (emprestimos != null)
            {
                if (emprestimosPeriodicos != null)
                {
                    emprestimos.AddRange(emprestimosPeriodicos);
                }
            }
            else if (emprestimosPeriodicos != null)
            {
                emprestimos = new List<Emprestimo>(emprestimosPeriodicos.Count);
                emprestimos.AddRange(emprestimosPeriodicos);
            }
            return emprestimos;
        }

        public List<Emprestimo> BuscarEmprestimosDevolvidos()
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from emprestimos where devolvido = 1");
            List<Emprestimo> emprestimos = BuscarEmprestimosFlex(cmd, 2);
            cmd = new SQLiteCommand("select * from emprestimos_periodicos where devolvido = 1");
            List<Emprestimo> emprestimosPeriodicos = BuscarEmprestimosFlex(cmd, 1);
            if (emprestimos != null)
            {
                if (emprestimosPeriodicos != null)
                {
                    emprestimos.AddRange(emprestimosPeriodicos);
                }
            }
            else if (emprestimosPeriodicos != null)
            {
                emprestimos = new List<Emprestimo>(emprestimosPeriodicos.Count);
                emprestimos.AddRange(emprestimosPeriodicos);
            }
            return emprestimos;
        }

        public List<Emprestimo> BuscarEmprestimosPendentes()
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from emprestimos where devolvido = 0");
            List<Emprestimo> emprestimos = BuscarEmprestimosFlex(cmd, 2);
            cmd = new SQLiteCommand("select * from emprestimos_periodicos where devolvido = 0");
            List<Emprestimo> emprestimosPeriodicos = BuscarEmprestimosFlex(cmd, 1);
            if (emprestimos != null)
            {
                if (emprestimosPeriodicos != null)
                {
                    emprestimos.AddRange(emprestimosPeriodicos);
                }
            }
            else if (emprestimosPeriodicos != null)
            {
                emprestimos = new List<Emprestimo>(emprestimosPeriodicos.Count);
                emprestimos.AddRange(emprestimosPeriodicos);
            }
            return emprestimos;
        }
    }
}
