﻿using GazBooksLibrary.Model;
using System;
using System.Collections.Generic;
using System.Data.SQLite;

namespace GazBooksLibrary.DAO
{
    public class LivroDAO
    {
        private String src = @"Data Source=C:\Users\Bruno\GazBooks\GazBooks.db";
        //private String src = @"Data Source=C:\Users\Cida\Desktop\gazbooks\GazBooks.db";

        public void InserirLivro(String autor, String nome, UInt16 edicao, String tags, UInt16 qtde)
        {
            String query = "insert into livros (autor, nome, edicao, tags, qtde) values (@autor, @nome, @edicao, @tags, @qtde)";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@autor", autor);
            cmd.Parameters.AddWithValue("@nome", nome);
            cmd.Parameters.AddWithValue("@edicao", edicao);
            cmd.Parameters.AddWithValue("@tags", tags);
            cmd.Parameters.AddWithValue("@qtde", qtde);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void DeletarLivro(UInt32 id)
        {
            String query = "delete from livros where id = @id";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void AtualizarQuantidade(UInt32 id, UInt16 qtde)
        {
            String query = "update livros set qtde = @qtde where id = @id";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@qtde", qtde);
            cmd.Parameters.AddWithValue("@id", id);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        private List<Livro> BuscarLivrosFlex(SQLiteCommand cmd)
        {
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            List<Livro> livros = null;
            using (var reader = cmd.ExecuteReader())
            {
                if (!reader.HasRows)
                {
                    return null;
                }
                livros = new List<Livro>();
                while (reader.Read())
                {
                    Livro l = new Livro();
                    l.ID = (UInt32)reader.GetInt32(0);
                    l.Autor = reader.GetString(1);
                    l.Titulo = reader.GetString(2);
                    l.Edicao = (UInt16)reader.GetInt16(3);
                    l.Qtde = (UInt16)reader.GetInt16(5);
                    l.Tags = reader.GetString(4).Split(';');
                    livros.Add(l);
                }
            }
            cmd.Dispose();
            con.Close();
            return livros;
        }

        public Livro BuscarLivroPorID(UInt32 id)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from livros where id = @id");
            cmd.Parameters.AddWithValue("@id", id);
            List<Livro> livros = BuscarLivrosFlex(cmd);
            return livros == null ? null : livros[0];
        }

        public List<Livro> BuscarTodosLivros()
        {
            return BuscarLivrosFlex(new SQLiteCommand("select * from livros"));
        }

        public List<Livro> BuscarLivrosPorNome(String nome)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from livros where nome like @nome");
            cmd.Parameters.AddWithValue("@nome", "%" + nome + "%");
            return BuscarLivrosFlex(cmd);
        }

        public List<Livro> BuscarLivrosPorAutor(String autor)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from livros where autor like @autor");
            cmd.Parameters.AddWithValue("@nome", "%" + autor + "%");
            return BuscarLivrosFlex(cmd);
        }

        public List<Livro> BuscarLivrosPorTags(String[] tags)
        {
            List<Livro> todosLivros = BuscarTodosLivros();
            List<Livro> livrosPesq = null;
            foreach (var livro in todosLivros)
            {
                foreach (var tag in tags)
                {
                    foreach (var tag2 in livro.Tags)
                    {
                        if (tag.Equals(tag2, StringComparison.CurrentCultureIgnoreCase))
                        {
                            if (livrosPesq == null)
                            {
                                livrosPesq = new List<Livro>(todosLivros.Count);
                            }
                            livrosPesq.Add(livro);
                        }
                    }
                }
            }
            return livrosPesq;
        }
    }
}
