﻿using GazBooksLibrary.Model;
using System;
using System.Collections.Generic;
using System.Data.SQLite;

namespace GazBooksLibrary.DAO
{
    public class PeriodicoDAO
    {
        private String src = @"Data Source=C:\Users\Bruno\GazBooks\GazBooks.db";
        //private String src = @"Data Source=C:\Users\Cida\Desktop\gazbooks\GazBooks.db";

        public void InserirPeriodico(String nome, String data_pub, String tags, UInt16 qtde)
        {
            String query = "insert into periodicos (nome, data_pub, tags, qtde) values (@nome, @data_pub, @tags, @qtde)";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@nome", nome);
            cmd.Parameters.AddWithValue("@data_pub", data_pub);
            cmd.Parameters.AddWithValue("@tags", tags);
            cmd.Parameters.AddWithValue("@qtde", qtde);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void DeletarPeriodico(UInt32 id)
        {
            String query = "delete from periodicos where id = @id";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@id", id);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        public void AtualizarQuantidade(UInt32 id, UInt16 qtde)
        {
            String query = "update periodicos set qtde = @qtde where id = @id";
            SQLiteCommand cmd = new SQLiteCommand(query);
            cmd.Parameters.AddWithValue("@qtde", qtde);
            cmd.Parameters.AddWithValue("@id", id);
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            SQLiteTransaction tr = con.BeginTransaction();
            cmd.ExecuteNonQuery();
            tr.Commit();
            tr.Dispose();
            cmd.Dispose();
            con.Close();
        }

        private List<Periodico> BuscarPeriodicosFlex(SQLiteCommand cmd)
        {
            SQLiteConnection con = new SQLiteConnection(src);
            cmd.Connection = con;
            con.Open();
            List<Periodico> periodicos = null;
            using (var reader = cmd.ExecuteReader())
            {
                if (!reader.HasRows)
                {
                    return null;
                }
                periodicos = new List<Periodico>();
                while (reader.Read())
                {
                    Periodico p = new Periodico();
                    p.ID = (UInt32)reader.GetInt32(0);
                    p.Nome = reader.GetString(1);
                    p.Data_Pub = reader.GetString(2);
                    p.Tags = reader.GetString(3).Split(';');
                    p.Qtde = (UInt16)reader.GetInt16(4);
                    periodicos.Add(p);
                }
            }
            cmd.Dispose();
            con.Close();
            return periodicos;
        }

        public Periodico BuscarPeriodicoPorID(UInt32 id)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from periodicos where id = @id");
            cmd.Parameters.AddWithValue("@id", id);
            List<Periodico> periodicos = BuscarPeriodicosFlex(cmd);
            return periodicos == null ? null : periodicos[0];
        }

        public List<Periodico> BuscarTodosPeriodicos()
        {
            return BuscarPeriodicosFlex(new SQLiteCommand("select * from periodicos"));
        }

        public List<Periodico> BuscarPeriodicosPorNome(String nome)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from periodicos where nome like @nome");
            cmd.Parameters.AddWithValue("@nome", "%" + nome + "%");
            return BuscarPeriodicosFlex(cmd);
        }

        public List<Periodico> BuscarPeriodicosPorData(String data)
        {
            SQLiteCommand cmd = new SQLiteCommand("select * from periodicos where data_pub like @data");
            cmd.Parameters.AddWithValue("@data", data);
            return BuscarPeriodicosFlex(cmd);
        }

        public List<Periodico> BuscarPeriodicosPorTags(String[] tags)
        {
            List<Periodico> todos = BuscarTodosPeriodicos();
            List<Periodico> periodicosPesq = null;
            foreach (var per in todos)
            {
                foreach (var tag in tags)
                {
                    foreach (var tag2 in per.Tags)
                    {
                        if (tag.Equals(tag2, StringComparison.CurrentCultureIgnoreCase))
                        {
                            if (periodicosPesq == null)
                            {
                                periodicosPesq = new List<Periodico>(todos.Count);
                            }
                            periodicosPesq.Add(per);
                        }
                    }
                }
            }
            return periodicosPesq;
        }
    }
}
